Scriptname AUAConfigQuestScript extends SKI_ConfigBase  

int OIDFollowersToggle
int OIDEquipSetsToggle

int OIDClearCustomAmmo
int OIDClearEquipSets

int OIDEquipSetList
int OIDArrowList
int OIDBoltList


bool bFollowers
bool bEquipSets
bool bClearCustomAmmo
bool bClearEquipSets
string[] EmptyArray
string[] EquipSetNames
string[] BowAmmoNames
string[] CrossbowAmmoNames


AUAQuestScript property AUA auto
AUAFollowerQuestScript property AUAFollower auto
GlobalVariable property AUAEquipSetsEnable auto

Event OnConfigInit()
	Pages = new string[1]
	Pages[0] = "$AUA_ConfigPageName"
EndEvent

Event OnConfigOpen()
	if !AUA.IsRunning()
		ShowMessage("$AUA_QuestError", false, "$AUA_OKButton")
	endif
	bFollowers = AUAFollower.IsRunning()
	bEquipSets = AUAEquipSetsEnable.GetValueInt() as bool
	bClearEquipSets = false
	bClearCustomAmmo = false
	EquipSetNames = AUA.GetEquipSetNames()
	BowAmmoNames = AUA.GetAmmoNamesByType(1)
	CrossbowAmmoNames = AUA.GetAmmoNamesByType(2)
EndEvent

Event OnConfigClose()
	; Use Equip Sets toggle
	if bEquipSets
		AUAEquipSetsEnable.SetValueInt(1)
	else
		AUAEquipSetsEnable.SetValueInt(0)
	endif
	; Follower Addon toggle
	if bFollowers && !AUAFollower.IsRunning()
		AUAFollower.Start()
	elseif !bFollowers && AUAFollower.IsRunning()
		AUAFollower.Stop()
	endif
	; Clear Equip Sets
	if bClearEquipSets
		AUA.DefineEquipSetArrays(true)
	endif
	; Clear Custom Ammo
	if bClearCustomAmmo
		AUA.RevertCustomAmmo(true)
	endif
	; Clear pointers so that arrays can be garbage collected
	EquipSetNames = EmptyArray
	BowAmmoNames = EmptyArray
	CrossbowAmmoNames = EmptyArray
EndEvent

Event OnPageReset(String Page)
	SetInfoText("")
	if Page == ""
		LoadCustomContent("AUA/AUA_logo.dds", 0, 68)
		return
	else
		UnloadCustomContent()
	endif

	if Page == Pages[0]
		SetCursorFillMode(TOP_TO_BOTTOM)

		AddHeaderOption("$AUA_ConfigHeader1")
		OIDEquipSetsToggle = AddToggleOption("$AUA_OptionTitle1", bEquipSets)
		OIDFollowersToggle = AddToggleOption("$AUA_OptionTitle2", bFollowers)
		
		AddHeaderOption("$AUA_ConfigHeader2")
		OIDClearEquipSets = AddToggleOption("$AUA_OptionTitle3", bClearEquipSets)
		OIDClearCustomAmmo = AddToggleOption("$AUA_OptionTitle4", bClearCustomAmmo)

		AddHeaderOption("$AUA_ConfigHeader3")
		OIDEquipSetList = AddMenuOption("$AUA_OptionTitle5", "")
		OIDArrowList = AddMenuOption("$AUA_OptionTitle6", "")
		OIDBoltList = AddMenuOption("$AUA_OptionTitle7", "")
	endif
EndEvent

Event OnOptionSelect(Int Option)
	if Option == OIDEquipSetsToggle
		bEquipSets = !bEquipSets
		SetToggleOptionValue(OIDEquipSetsToggle, bEquipSets)
	elseif Option == OIDFollowersToggle
		bFollowers = !bFollowers
		SetToggleOptionValue(OIDFollowersToggle, bFollowers)
	elseif Option == OIDClearEquipSets
		bClearEquipSets = !bClearEquipSets
		SetToggleOptionValue(OIDClearEquipSets, bClearEquipSets)
	elseif Option == OIDClearCustomAmmo
		bClearCustomAmmo = !bClearCustomAmmo
		SetToggleOptionValue(OIDClearCustomAmmo, bClearCustomAmmo)
	endif
EndEvent

Event OnOptionMenuOpen(Int Option)
	if Option == OIDEquipSetList
		SetMenuDialogOptions(EquipSetNames)
		;SetMenuDialogStartIndex(0)
		;SetMenuDialogDefaultIndex(0)
	elseif Option == OIDArrowList
		SetMenuDialogOptions(BowAmmoNames)
		;SetMenuDialogStartIndex(0)
		;SetMenuDialogDefaultIndex(0)
	elseif Option == OIDBoltList
		SetMenuDialogOptions(CrossbowAmmoNames)
		;SetMenuDialogStartIndex(0)
		;SetMenuDialogDefaultIndex(0)
	endif
EndEvent

Event OnOptionDefault(Int Option)
	if Option == OIDEquipSetsToggle
		bEquipSets = false
		SetToggleOptionValue(OIDEquipSetsToggle, false)
	elseif Option == OIDFollowersToggle
		bFollowers = false
		SetToggleOptionValue(OIDFollowersToggle, false)
	elseif Option == OIDClearEquipSets
		bClearEquipSets = false
		SetToggleOptionValue(OIDClearEquipSets, false)
	elseif Option == OIDClearCustomAmmo
		bClearCustomAmmo = false
		SetToggleOptionValue(OIDClearCustomAmmo, false)
	endif
EndEvent

Event OnOptionHighlight(Int Option)
	if Option == OIDEquipSetsToggle
		SetInfoText("$AUA_InfoText1")
	elseif Option == OIDFollowersToggle
		SetInfoText("$AUA_InfoText2")
	elseif Option == OIDClearEquipSets
		SetInfoText("$AUA_InfoText3")
	elseif Option == OIDClearCustomAmmo
		SetInfoText("$AUA_InfoText4")
	elseif Option == OIDEquipSetList
		SetInfoText("$AUA_InfoText5")
	elseif Option == OIDArrowList
		SetInfoText("$AUA_InfoText6")
	elseif Option == OIDBoltList
		SetInfoText("$AUA_InfoText7")
	else
		SetInfoText("")
	endif
EndEvent
