Scriptname AUAPlayerAliasScript extends ReferenceAlias

import Utility

AUAQuestScript property AUA auto
AUAFollowerQuestScript property AUAFollower auto
GlobalVariable property AUAEquipSetsEnable auto
Spell property AUABoundWeaponCheckSpell auto
MagicEffect property AUABoundWeaponCheckEffect auto

Actor PlayerRef
Ammo EquippedAmmo
Ammo LastUsedArrows
Ammo LastUsedBolts
bool SkipUpdate

Event OnPlayerLoadGame()
	AUA.GameLoaded = true
	AUAFollower.GameLoaded = true
EndEvent

Event OnInit()
	RegisterForSingleUpdate(1)
EndEvent

Auto State Init
	Event OnUpdate()
		PlayerRef = Game.GetPlayer()
		EquippedAmmo = AUA.GetEquippedAmmo(PlayerRef)
		if EquippedAmmo
			if PlayerRef.GetEquippedItemType(1) != 7 && PlayerRef.GetEquippedItemType(1) != 12
				PlayerRef.UnequipItem(EquippedAmmo, false, true)
			endif
		endif

		GoToState("Run")
	EndEvent
EndState

State Run
	Event OnObjectEquipped(Form akBaseObject, ObjectReference akReference)
		if (akBaseObject as ammo)
			; remember ammo to be auto-unequipped
			EquippedAmmo = akBaseObject as ammo
			; skip if auto-equipped
			if SkipUpdate
				UnregisterForUpdate()
				SkipUpdate = false
				return
			endif
			; evaluate equipped ammo after a short delay
			RegisterForSingleUpdate(1)
		elseif (akBaseObject as weapon)
			; bound weapon check
			if PlayerRef.HasMagicEffect(AUABoundWeaponCheckEffect)
				PlayerRef.DispelSpell(AUABoundWeaponCheckSpell)
			endif
			AUABoundWeaponCheckSpell.Cast(PlayerRef)
			if PlayerRef.HasMagicEffect(AUABoundWeaponCheckEffect)
				return
			endif
			; bow equipped
			if PlayerRef.GetEquippedItemType(1) == 7
				; equip set override
				if AUAEquipSetsEnable.GetValueInt()
					ammo EquipSetAmmo = AUA.GetEquipSetAmmo(PlayerRef.GetEquippedWeapon())
					if EquipSetAmmo
						if PlayerRef.GetItemCount(EquipSetAmmo)
							LastUsedArrows = EquipSetAmmo
						endif
					endif
				endif
				; try to equip last used ammo
				if LastUsedArrows
					if PlayerRef.GetItemCount(LastUsedArrows)
						if !PlayerRef.IsEquipped(LastUsedArrows)
							SkipUpdate = true
							PlayerRef.EquipItem(LastUsedArrows, false, true)
						endif
					endif
				endif
			; crossbow equipped
			elseif PlayerRef.GetEquippedItemType(1) == 12
				; equip set override
				if AUAEquipSetsEnable.GetValueInt()
					ammo EquipSetAmmo = AUA.GetEquipSetAmmo(PlayerRef.GetEquippedWeapon())
					if EquipSetAmmo
						if PlayerRef.GetItemCount(EquipSetAmmo)
							LastUsedBolts = EquipSetAmmo
						endif
					endif
				endif
				; try to equip last used ammo
				if LastUsedBolts
					if PlayerRef.GetItemCount(LastUsedBolts)
						if !PlayerRef.IsEquipped(LastUsedBolts)
							SkipUpdate = true
							PlayerRef.EquipItem(LastUsedBolts, false, true)
						endif
					endif
				endif
			endif
		endif
	EndEvent

	Event OnObjectUnequipped(Form akBaseObject, ObjectReference akReference)
		if (akBaseObject as weapon) && EquippedAmmo
			if PlayerRef.GetEquippedItemType(1) != 7 && PlayerRef.GetEquippedItemType(1) != 12
				if IsInMenuMode()
					Wait(0.01)
				endif
				if PlayerRef.IsEquipped(EquippedAmmo)
					PlayerRef.UnequipItem(EquippedAmmo, false, true)
				endif
			endif
		endif
	EndEvent

	Event OnUpdate()
		Ammo CurrentAmmo = EquippedAmmo
		if PlayerRef.IsEquipped(CurrentAmmo)
			int AmmoType = AUA.GetAmmoType(CurrentAmmo)
			if AmmoType == 1
				LastUsedArrows = CurrentAmmo
			elseif AmmoType == 2
				LastUsedBolts = CurrentAmmo
			endif
			; create equip set
			if AUAEquipSetsEnable.GetValueInt()
				if (PlayerRef.GetEquippedItemType(1) == 7 && AmmoType == 1) || (PlayerRef.GetEquippedItemType(1) == 12 && AmmoType == 2)
					AUA.CreateEquipSet(PlayerRef.GetEquippedWeapon(), CurrentAmmo)
				endif
			endif
		endif
	EndEvent
EndState
